'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.EitherLeft = exports.EitherRight = undefined;
exports.Right = Right;
exports.Left = Left;
exports.of = of;
exports.is = is;

var _base = require('./base');

var _base2 = _interopRequireDefault(_base);

require('./index.h');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

class EitherRight extends _base2.default {
  constructor(value) {
    super();
    this.value = value;
    /*:: return this */
  }
  isRight() {
    return true;
  }
  isLeft() {
    return false;
  }

  orElse() {
    return this.value;
  }

  promise() {
    return Promise.resolve(this.value);
  }

  fold(r /*:: , l: (x: L) => O*/) {
    return r(this.value);
  }

  swap() {
    return new EitherLeft(this.value);
  }

  equals(value) {
    if (value instanceof _base2.default) {
      if (value instanceof EitherRight) {
        return value.value === this.value;
      } else return false;
    } else return value === this.value;
  }

  // --- Tap section ---

  tap(fn) {
    return this.tapR(fn);
  }
  tapR(fn) {
    fn(this.value);
    return this;
  }
  tapL() {
    return this;
  }
  bitap(r /*:: , l: (x: L) => any */) {
    return this.tapR(r);
  }

  // --- Map section ---

  map(fn) {
    return this.mapR(fn);
  }
  mapR(fn) {
    return new EitherRight(fn(this.value));
  }
  mapL() {
    return (/*:: changeL( */this
    ); /*::) */
  }
  bimap(r /*:: , l: (x: L) => L1 */) {
    return (/*:: changeL( */this.mapR(r)
    ); /*::) */
  }

  // --- Chain section ---

  chain(fn) {
    return this.chainR(fn);
  }
  chainR(fn) {
    return (/*:: concatL(*/fn(this.value)
    ); /*::) */
  }
  chainL() {
    return (/*:: changeL(concatR(*/this
    ); /*::)) */
  }
  bichain(r
  /*::, l: (x: L) => Apropos<R2, L2> */
  ) {
    return (/*:: concatR(concatL(*/r(this.value)
    ); /*::)) */
  }

  // --- Cond section ---

  cond(fn) {
    return fn(this.value);
  }

  chainCond(cond, pass, fail) {
    if (this.cond(cond)) return (/*:: concatL( */this.mapR(pass)
    ); /*::) */
    else return (/*:: changeR( */new EitherLeft(fail(this.value))
      ); /*::) */
  }

  logic({
    cond, pass, fail
  }) {
    return this.chainCond(cond, pass, fail);
  }

  // --- Combine section ---

  alt() {
    return (/*:: concatR(changeL( */this
    ); /*::)) */
  }

  and(e) {
    return (/*:: concatL(*/e
    ); /*::) */
  }

  thru(fn) {
    return fn(this);
  }

  ap(e) {
    if (e.isRight()) return (/*:: concatL(*/e /*::) */.mapR(fn => fn(this.value))
    );
    return (/*:: changeR(concatL( */e
    ); /*::)) */
  }

  toString() {
    return 'Right( ' + failSafeStringify(this.value) + ' )';
  }

  inspect() {
    return this.toString();
  }
}

exports.EitherRight = EitherRight;
class EitherLeft extends _base2.default {
  constructor(value) {
    super();
    this.value = value;
    /*:: return this */
  }
  isRight() {
    return false;
  }
  isLeft() {
    return true;
  }

  orElse(value) {
    return value;
  }

  promise() {
    return Promise.reject(this.value);
  }

  fold(r, l) {
    return l(this.value);
  }

  swap() {
    return new EitherRight(this.value);
  }

  equals(value) {
    if (value instanceof _base2.default) {
      if (value instanceof EitherLeft) {
        return value.value === this.value;
      } else return false;
    } else return value === this.value;
  }

  // --- Tap section ---

  tap(fn) {
    return this.tapR(fn);
  }
  tapR() {
    return this;
  }
  tapL(fn) {
    fn(this.value);
    return this;
  }
  bitap(r, l) {
    return this.tapL(l);
  }

  // --- Map section ---

  map() {
    return (/*:: changeR( */this
    ); /*::) */
  }
  mapR() {
    return (/*:: changeR( */this
    ); /*::) */
  }
  mapL(fn) {
    return new EitherLeft(fn(this.value));
  }
  bimap(r, l) {
    return (/*:: changeR( */this.mapL(l)
    ); /*::) */
  }

  // --- Chain section ---

  chain(fn) {
    return this.chainR(fn);
  }
  chainR() {
    return (/*:: changeR(concatL(*/this
    ); /*::)) */
  }
  chainL(fn) {
    return (/*:: concatR(*/fn(this.value)
    ); /*::) */
  }
  bichain(r, l) {
    return (/*:: concatR(concatL(*/l(this.value)
    ); /*::)) */
  }

  // --- Cond section ---

  cond() {
    return false;
  }

  chainCond() {
    return (/*:: changeR(concatL(*/this
    ); /*::)) */
  }

  logic() {
    return (/*:: changeR(concatL(*/this
    ); /*::)) */
  }

  // --- Combine section ---

  alt(e) {
    return (/*:: concatR(*/e
    ); /*::) */
  }

  and() {
    return (/*:: changeR(concatL( */this
    ); /*::)) */
  }

  thru(fn) {
    return fn(this);
  }

  ap() {
    return (/*:: changeR(concatL( */this
    ); /*::)) */
  }

  toString() {
    return 'Left( ' + failSafeStringify(this.value) + ' )';
  }

  inspect() {
    return this.toString();
  }
}

exports.EitherLeft = EitherLeft;
(0, _base.makeRight)(EitherRight);
(0, _base.makeLeft)(EitherLeft);

function failSafeStringify(value) {
  try {
    return JSON.stringify(value);
  } catch (error) {
    return String(value);
  }
}

/**
 * Create right-handed value.
 *
 * Left-handed type is inferred from usage
 *
 * @template R
 * @template L
 * @param {R} value
 * @returns {Apropos<R, L>}
 */
function Right(value) {
  return new EitherRight(value);
}

/**
 * Create left-handed value.
 *
 * Right-handed type is inferred from usage
 *
 * @template R
 * @template L
 * @param {L} value
 * @returns {Apropos<R, L>}
 */
function Left(value) {
  return new EitherLeft(value);
}

/**
 * Create pure right-handed value
 *
 * Left-handed type is empty
 *
 * @template R
 * @param {R} value
 * @returns {Apropos<R, void>}
 */
function of(value) {
  return new EitherRight(value);
}

/**
 * Checks whether an object is an instance of `Apropos`
 *
 * @template T
 * @param {T} value
 * @returns {boolean}
 */
function is(value) {
  return value instanceof _base2.default;
}
//# sourceMappingURL=isomer.js.map